package cz.muni.fi.cepportal.cepserver;

import com.espertech.esper.client.EPServiceProvider;
import com.espertech.esper.client.EPServiceProviderManager;
import cz.muni.fi.cepportal.cepserver.serviceprovider.EsperServiceProvider;
import cz.muni.fi.cepportal.jms.JMSContext;
import cz.muni.fi.cepportal.jms.JMSContextFactory;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.rmi.registry.LocateRegistry;
import java.util.Properties;
import javax.jms.JMSException;
import javax.jms.MessageConsumer;
import javax.management.InstanceAlreadyExistsException;
import javax.management.MBeanRegistrationException;
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.MalformedObjectNameException;
import javax.management.NotCompliantMBeanException;
import javax.management.ObjectName;
import javax.management.remote.JMXConnectorServer;
import javax.management.remote.JMXConnectorServerFactory;
import javax.management.remote.JMXServiceURL;
import javax.naming.NamingException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 *  CEP Server Main class.
 *  This class initialize engine Esper, start RMI registry, start JMX server and connect to JMS server.
 * 
 *  @author Štefan Repček
 */
public class CEPServerMain 
{
    
     private static Log log = LogFactory.getLog(CEPServerMain.class);
     private static boolean isShutdown;
    
    public static void main( String[] args ) throws IOException, MalformedObjectNameException, InstanceAlreadyExistsException, MBeanRegistrationException, NotCompliantMBeanException, JMSException, NamingException, InterruptedException
    {
         log.info("Loading properties");
        Properties properties = new Properties();
        InputStream propertiesIS = CEPServerMain.class.getClassLoader().getResourceAsStream(ServerConstants.CONFIG_FILENAME);
        if (propertiesIS == null)
        {
            throw new RuntimeException("Properties file '" + ServerConstants.CONFIG_FILENAME + "' not found in classpath");
        }
        properties.load(propertiesIS);
        
        
        log.info("Starting RMI registry");
        int port = Integer.parseInt(properties.getProperty(ServerConstants.RMI_PORT));
        LocateRegistry.createRegistry(port);
        
        
        log.info("Obtaining JMX server and connector");
        MBeanServer mbs = MBeanServerFactory.createMBeanServer();
        String jmxServiceURL = "service:jmx:rmi:///jndi/rmi://localhost:" + port + "/server";
        JMXServiceURL jmxURL = new JMXServiceURL(jmxServiceURL);
        JMXConnectorServer cs = JMXConnectorServerFactory.newJMXConnectorServer(jmxURL, null, mbs);
        cs.start();
        
        
        log.info("Getting Esper engine instance");
        EPServiceProvider engine = EPServiceProviderManager.getDefaultProvider();
        
        
       
        
        
        log.info("Connecting to JMS server");
        String factory = properties.getProperty(ServerConstants.JMS_CONTEXT_FACTORY);
        String jmsProviderPort = properties.getProperty(ServerConstants.JMS_PROVIDER_PORT);
        String jmsProviderURLOptions = properties.getProperty(ServerConstants.JMS_PROVIDER_URL_OPTIONS);
        String jmsurl = "tcp://localhost:" + jmsProviderPort + jmsProviderURLOptions;
        String connFactoryName = properties.getProperty(ServerConstants.JMS_CONNECTION_FACTORY_NAME);
        String user = properties.getProperty(ServerConstants.JMS_USERNAME);
        String password = properties.getProperty(ServerConstants.JMS_PASSWORD);
        String serverDestination = properties.getProperty(ServerConstants.JMS_SERVER_DESTINATION);
        String clientDestination = properties.getProperty(ServerConstants.JMS_CLIENT_DESTINATION);
        boolean isTopic = Boolean.parseBoolean(properties.getProperty(ServerConstants.JMS_IS_TOPIC));
        JMSContext jmsCtx = JMSContextFactory.createContext(factory, jmsurl, connFactoryName, user, password, serverDestination, clientDestination, isTopic);
        
        
      
        jmsCtx.getConnection().start();
        ServerResponseMessager responseMessager = new ServerResponseMessager(jmsCtx);
        XMLEventMessageListener messageListener = new XMLEventMessageListener(engine, responseMessager);
        MessageConsumer consumer = jmsCtx.getSession().createConsumer(jmsCtx.getServerDestination());
        consumer.setMessageListener(messageListener);
        
        log.info("Registering MBean");
        ObjectName name = new ObjectName(ServerConstants.MBEAN_NAME);
        EsperServiceProvider mbean = new EsperServiceProvider(engine, responseMessager);
        mbs.registerMBean(mbean, name);
    
        Runtime.getRuntime().addShutdownHook(new Thread()
        {
            @Override
            public void run()
            {
                isShutdown = true;
            }
        }); 
    
    
    InputStreamReader inputStreamReader = new InputStreamReader (System.in);
    BufferedReader stdin = new BufferedReader (inputStreamReader);
     log.info("Server is running");
     do
        {
         if(stdin.ready());  
         {
         String line  = stdin.readLine();   
         isShutdown = line.trim().equalsIgnoreCase("exit");
         }
       
         
           
        }
     while(!isShutdown);
     
      log.info("Shutting down server");
        jmsCtx.destroy();
        
        System.exit(0);
    }
}
