/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.muni.fi.cepportal.cepserver.serviceprovider;

import java.io.IOException;



/**
 * MBEAN interface
 * @author Štefan Repček
 */
public interface EsperServiceProviderMBean {
     /**
      *  Creates and starts an EPL statement or Pattern statement
      * 
      * @param expression - represents statement,  must follow the documented syntax for EPL or pattern statements
      * @param statementName - is the name to assign to the statement for use in managing the statement
      * @param listener - is listener, which is assigned to this statement
      * @param isPattern - defines if cep server has to create epl statement or pattern statement, 
      *                  if is false - creates EPL statement else create pattern statement
      * @throws IOException - is throwed when communication problem appears
      */
     public void createEPL(String expression, String statementName, String listener, boolean isPattern) throws IOException;
     /**
      *  Registers event type in engine Esper
      * @param xsdScheme - XML Scheme of event type
      * @param rootElementName - root element name of even type
      * @param eventName -  name of event types
      * @throws IOException - is throwed when communication problem appears
      */
     public void registerEventType(String xsdScheme,String rootElementName, String eventName) throws IOException;
     /**
      *  Stops and Destroys EPL or pattern statement
      * @param statementName - name of statement which has to be destroyed
      * @throws IOException - is throwed when communication problem appears
      */
     public void destroyEPL(String statementName) throws IOException;
     /**
      * Stops and destroys all actual statements.
      * @throws IOException - is throwed when communication problem appears
      */
     public void destroyAllStatements() throws IOException;
     /**
      * Removes an event type by its name
      * @param eventName -  name of event type which has to be removed
      * @throws IOException - is throwed when communication problem appears
      */
     public void removeEventType(String eventName) throws IOException;
     /**
      *  Restars engine Esper
      * @throws IOException - is throwed when communication problem appears
      */
     public void restartEngine() throws IOException;
  
    
}
