/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.muni.fi.cepportal.jms;

import java.util.Hashtable;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.Session;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

/**
 * This class creates connection to JMS provider and returns JMXContext
 * 
 * @author Štefan Repček
 */
public abstract class JMSContextFactory
{
	public static JMSContext createContext(String ctxFactory, String url, String connectionFactoryName, String user, String pwd, String jmsServerDestination, String jmsClientDestination, boolean isTopic) throws JMSException, NamingException
    {
        ConnectionFactory factory = getConnectionFactory(ctxFactory, url, connectionFactoryName, user, pwd);
        Connection connection = factory.createConnection(user, pwd);
        Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);

        Destination serverDestination;
        Destination clientDestination;
        if (isTopic)
        {
            serverDestination = session.createTopic(jmsServerDestination);
            clientDestination = session.createTopic(jmsClientDestination);
        }
        else
        {
            serverDestination = session.createQueue(jmsServerDestination);
            clientDestination = session.createQueue(jmsClientDestination);
        }

		return new JMSContext(factory, connection, session, serverDestination, clientDestination);
	}

	@SuppressWarnings("unchecked")
	private static ConnectionFactory getConnectionFactory(String ctxFactory, String url, String connectionFactoryName, String user, String pwd) throws NamingException
    {
		Hashtable env = new Hashtable();
		env.put(Context.INITIAL_CONTEXT_FACTORY, ctxFactory);
		env.put(Context.PROVIDER_URL, url);
		env.put(Context.SECURITY_PRINCIPAL, user);
		env.put(Context.SECURITY_CREDENTIALS, pwd);
		InitialContext jndiContext = new InitialContext(env);
		return (ConnectionFactory)jndiContext.lookup(connectionFactoryName);
	}



}
